function [ber_demod] = mil110CDSimulador(EsNodB, seed)
%------------------------------------------------------
%Estrutura Geral do Simulador MIL-STD-188-110C Anexo D:
%------------------------------------------------------
%TRANSMISSOR: (bits) --> CODIFICADOR CONVOLUCIONAL --> INTERLEAVER --> MAPEADOR GRAY --> SCRAMBLER --> --> MODULADOR PSK --> INSERCAO DE PREAMBULO/MINIPROBES --> FORMATADOR PULSO --> (entrada do canal)
%
%CANAL HF: (saida do transmissor) --> MODELO WATTERSON --> RUIDO AWGN --> (entrada do receptor)
%
%RECEPTOR HARD: (saida do canal) --> (filtro recepcao)  --> (sincronizador quadro/simbolo) --> (equalizacao) --> (REMOCAO DE PREAMBULO/MINIPROBES + DESCRAMBLER) --> DEMODULADOR --> DEMAPEADOR GRAY --> DEINTERLEAVER --> (entrada do decodificador 'hard')
%
%RECEPTOR SOFT: (saida do canal) --> (filtro recepcao)  --> (sincronizador quadro/simbolo) --> (equalizacao) --> (REMOCAO DE PREAMBULO/MINIPROBES + DESCRAMBLER) --> (formacao de 'soft' bits) --> DEINTERLEAVER --> (entrada do decodificador 'soft')

%-------------------------
%MIL-STD-188-110C Anexo D:
%-------------------------
%Variavel que ajusta o tamanho do bloco de interleaving (entrelacamento).
%Possui 4 opcoes: 'ultrashort', 'short', 'medium' e 'long';
varMil110CD.modointer  = 'long';

%Variavel que ajusta o numero de repeticoes da subseção de sincronizacao do
%preambulo. Pode assumir valores inteiros entre 1 e 32;
varMil110CD.M          = 1;                         %Opções: 1:32;

%Variavel que ajusta o numero de repeticoes da subsecao TLC (transmitter
%level control) do preambulo. Pode assumir valores inteiros entre 0 e 255;
varMil110CD.N_tlc      = 1;

%Variavel que ajusta o tamanho da parte 'fixa' da subseção de sincronizacao
%do preambulo. Pode assumir dois valores : 1 ou 9;
varMil110CD.N_fixa     = 1;

%Variavel que ajusta o 'constraint length' do codificador convolucional,
%definido aqui como o numero de registros de memoria do codificador-mãe,
%incluindo o bit de entrada no instante atual. Pode assumir dois valores: 7
%ou 9;
varMil110CD.codLength  = 7;

%Variavel que ajusta o numero de blocos de interleaving transmitidos;
varMil110CD.nFrames    = 1;

%Variavel correspondente a semente que controla o gerador aleatorio
%utilizado pelo MATLAB. Esta funcionalidade permite garantir que toda
%transmissao independente (realizacao), empregando as mesmas variaveis de
%ajuste e a mesma semente, gere exatamente o mesmo resultado.
varMil110CD.seed    = seed;

%--------------------------
%Codificador convolucional:
%--------------------------
% O codificador convolucional utilizado por este simulador corresponde a um
%codigo proprietario. O Anexo D da norma MIl-STD-188-110C utiliza um método
%de terminacao especifica, em que os primeiros 'codLength-1' bits de cada
%bloco de interleaving sao usados para pre-preenchimento dos registros de
%memoria do codificador, apos o qual uma copia deles e anexada ao final do
%bloco. Infelizmente, o objeto 'comm.convolutionalEncoder' do MATLAB nao
%possui esse metodo de terminacao disponivel.

%-------
%Filtro:
%-------
%Variavel que ajusta o fator de rolloff do filtro formatador de pulso no
%transmissor;
varShapping.rolloffFactor      = 0.35;

%Variavel que ajusta o atraso (em simbolos) do filtro formatador de pulso;
varShapping.pulseDelay         = 5;

%Variavel que ajusta o fator de superamostragem implementado pelo filtro
%formatador de pulso. Retornara um erro se o valor adotado nao for tal que
%o produto 'RER x 19200 x oversamplingFactor' resulte um inteiro (sendo RER e
%o atraso (em segundos) entre os raios do canal Watterson);
varShapping.oversamplingFactor = 10;

%-----------------
%Watterson + AWGN:
%-----------------
%Variavel que seleciona o modelo de canal Watterson. Sao 4 opcoes, cada uma
%equivalendo a pares de valores de espalhamento de atraso (RER) e
%espalhamento Doppler (ED) distintos: canal 'bom' (RER = 0.0005s e ED =
%0.1Hz), canal 'moderado' (RER = 0.001s e ED = 0.5Hz), canal 'ruim' (RER =
%0.002s e ED = 1Hz) e canal 'muito_ruim' (RER = 0.005s e ED = 5Hz);
varWatterson.model  = 'ruim';

%Variavel que ajusta a razao sinal-ruido, em decibeis. Corresponde a razao
%entre a energia media do simbolo PSK transmitido (Es) e a densidade
%espectral de potencia do ruido (No);
varWatterson.EsNodB = EsNodB;

%Variavel correspondente a semente que controla o gerador aleatorio
%utilizado pelo MATLAB. Esta funcionalidade permite garantir que toda
%transmissao independente (realizacao), empregando as mesmas variaveis de
%ajuste e a mesma semente, gere exatamente o mesmo resultado.
varWatterson.seed = seed;

%----------
%Simulador:
%----------
%mil110CDParametros = carrega parametros gerais da forma de onda selecionada.
%Retorna um struct 'mil110CD' que orienta o funcionamento dos demais componentes
%da cadeia de transmissao:
mil110CD          = mil110CDParametros(varMil110CD);

%mil110CDTransmissor = simula o transmissor MIL-STD-188-110C Anexo D. Retorna
%uma struct 'TxOut' com os seguintes vetores:
%
%BitInCodIn = bits de dados transmitidos. Entrada do codificador
%convolucional. Seu tamanho dependera do tamanho do bloco de interleaving
%escolhido: multiplos de 3264 bits (para 'ultrashort'), multiplos de 13056
%('short'), multiplos de 52224 ('medium') e multiplos de 208896 ('long');
%
%CodOutInterIn = bits de saida do codificador convolucional e de entrada
%do interleaver. Como o codigo convolucional utiliza taxa 3/4 para a forma
%de onda numero 6/24kHz, o tamanho deste vetor sera 4/3 do tamanho de
%'BitInCodIn';
%
%InterOutMapIn = bits de saida do interleaver e de entrada do mapeador
%Gray. Seu tamanho é o mesmo do vetor 'CodOutInterIn';
%
%MapOutScrambIn = simbolos PSK, inteiros, de saida do mapeados Gray e de
%entrada do scrambler (embaralhador). Como a forma de onda numero 6 utiliza
%modulacao 4-PSK, o tamanho deste vetor sera a metade de 'InterOutMapIn ';
%
%ScrambOutMultiplexIn = simbolos PSK, inteiros, de saida do scrambler e de
%entrada do formador de quadro (multiplexador). Note que o embaralhador
%converte todos os simbolos, que eram 4-PSK, para 8-PSK, para serem assim
%modulados e transmitidos pelo canal. Seu tamanho sera o mesmo de 'MapOutScrambIn';
%
%ComplexPSKSymbol = corresponde a versao modulada, em simbolos PSK
%complexos, do TxOut.MapOutMultiplexIn;
%
%ModConstellation = constelacao PSK utilizada para modular os simbolos de
%entrada do modulador;
%
%ProbeInfo = vetor que indica a distribuicao/posicao de probes ao longo do quadro
%transmitido ('1' indica que o simbolo naquela posicao e conhecido (probe),
%enquanto '0' indica que o simbolo naquela posicao e desconhecido no receptor);
%

%ProbeInfo = vetor binário que indica a distribuicao/posicao de probes ao
%longo do quadro transmitido ('1' indica que o simbolo naquela posicao e
%conhecido (probe), enquanto '0' indica que o simbolo naquela posicao e
%desconhecido no receptor). As primeiras posicoes correspondem ao
%preambulo, cujo comprimento e calculado da seguinte forma:
%
%                N_tlc x 256 + M x (N_fixa + 4 + 5) x 256,
%
%onde N_tlc, M e N_fixa sao variaveis presentes na 'struct' varMil110CD.
%Apos este comprimento, há ainda um quadro de probes de treinamento.
%Finalmente, as posicoes seguintes correspondem a intercalacoes de quadros
%de dados com dados de probes de treinamento. Para a forma de onda numero
%6/24kHz, o tamanho dos quadros de dados e de 2176 simbolos, enquanto o dos
%de probes e de 272 simbolos;
%
%MultiplexOut = saida do transmissor. Simbolos PSK, complexos, de saida
%do modulador, incluindo preambulo, probes e dados. Entrada do modulo
%formatador de pulso (segundo a norma, raiz de coseno levantado). Seu
%tamanho e o mesmo do vetor 'ProbeInfo';
TxOut            = mil110CDTransmissor(mil110CD, varMil110CD.nFrames, varMil110CD.seed);

%mil110CDFiltroTransmissor = simula o siltro de saida do transmissor
%(formatador de pulso). Retorna um struct 'ShappingOut' com as seguintes
%saidas:
%
%filter = filtro raiz de coseno levantado utilizado na formatacao dos
%pulsos. Seu tamanho se da pela relacao '2 x pulseDelay x oversamplingFactor + 1';
%
%out = dados de saida do formatador de pulso. Pulsos ocupando
%largura de banda (1 + rolloffFactor) x 9600 (banda base). Seu
%tamanho corresponde ao tamanho do vetor 'MultiplexOut' multiplicado pelo
%fator de superamostragem (oversamplingFactor), acrescido ainda das
%amostras de retardo (pulseDelay x oversamplingFactor) introduzidas pelo
%filtro no começo e no fim do vetor;
ShappingOut      = mil110CDFiltroTransmissor(TxOut.MultiplexOut, varShapping);

%mil110CDCanalWatterson = simula o canal HF ionosferico, segundo o modelo de
%canal proposto por (Watterson, 1970). Retorna um struct 'WattersonOut' com
%as seguintes saidas:
%
%raios = matriz com 2 colunas, cada uma representando um dos raios 'Watterson'
%ao longo da transmissao do sinal. Seu tamanho e o mesmo do vetor
%'ShappingOut.out';
%
%RER = atraso entre os raios, em segundos;
%
%ED = espalhamento Doppler dos raios, em Hz;
%
%WattersonOutAwgnIn = sinal convoluido com os raios e, portanto, com presenca
%de ISI. Seu tamanho corresponde ao do vetor 'ShappingOut.out', acrescido
%do atraso (em amostras) entre os dois raios do canal Watterson (RER x
%19200 x oversamplingFactor);
%
%noise = ruido AWGN adicionado ao sinal 'WattersonOutAwgnIn'. Seu tamanho e
%o mesmo do vetor 'WattersonOutAwgnIn';
%
%AwgnOut = dados de saida do emulador de canal Watterson.
%Corresponde ao sinal com presenca de ISI e contaminado com AWGN. Seu
%tamanho e o mesmo do vetor 'WattersonOutAwgnIn';
WattersonOut     = mil110CDCanalWatterson(ShappingOut.out, varWatterson, varShapping);

%mil110CDFiltroCasado = simula o filtro de recepcao. Para este simulador
%basico proposto, trata-se de um filtro casado ao filtro formatador de
%pulso. Retorna um struct 'MatchedFilterOut' com as seguintes saidas:
%
%filter = filtro utilizado na filtragem do sinal de recepcao. Neste caso,
%ele e semelhante ao 'ShappingOut.filter';
%
%out = dados de saida do filtro de recepcao. Neste caso, corresponde ao sinal
%confinado novamente na largura de banda (1 + rolloffFactor) x 9600
%(banda base). Seu tamanho corresponde ao tamanho do vetor 'AwgnOut'
%acrescido das amostras de retardo introduzidas pelo filtro no começo e no
%final do vetor;
MatchedFilterOut = mil110CDFiltroCasado(WattersonOut.AwgnOut, varShapping);
%MatchedFilterOut = mil110CDFiltroCasado(ShappingOut.out, varShapping);

%mil110CDSincronismoQuadro = simula um sincronizador de quadro simples. O modelo
%Watterson para o canal HF é um filtro FIR com 2 componentes principais.
%Para recuperar o sinal transmitido, o receptor precisa sincronizar a cadeia
%de recepção em um desses dois componentes. Esse papel é realizado por alguma
%técnica de sincronismo de quadro, no início da recepção do sinal. Para este
%trabalho, considerou-se que o sincronismo de quadro encontra o componente de
%maior módulo e sincroniza a cadeia de recepção em torno deste cursor. O outro
%componente passa a ser um pós-cursor (caso o primeiro componente seja sincronizado)
%ou um pré-cursor (caso o segundo o seja). Este sincronizador simples
%retorna um numero, que e uma estimativa da amostra onde se identifica o
%comeco da transmissao. Na ausencia de canal, ele retornara o valor
%correspondente às amostras de retardo dos filtros de transmissao e de
%recepcao, mais um;
FrameStartSample = mil110CDSincronismoQuadro(MatchedFilterOut.out, mil110CD, varShapping);

%mil110CDSincronismoSimbolo = simula um sincronizador de simbolo (ponto de
%amostragem) simples. Neste caso, ele apenas recebe o ponto de inicio da
%transmissao e amostra o sinal de saida do filtro de recepcao em intervalos
%de 'varShapping.oversamplingFactor' amostras. Retorna um struct 'SyncSymbolOut'
%com a seguinte saida:
%
%out = dados de saida do sincronizador de simbolo. Corresponde a um vetor de
%simbolos PSK, complexos. Na ausencia de canal, deve ser identico ao vetor
%'TxOut.MultiplexOut'. Na presenca de canal, sera uma versao distorcida de
%'MultiplexOut', e o seu tamanho correspondera ao de 'MultiplexOut'
%acrescido de alguns simbolos no final, correspondentes ao espalhamento de
%atraso provocado pelo canal;
SyncSymbolOut    = mil110CDSincronismoSimbolo(MatchedFilterOut.out, FrameStartSample, varShapping);

%mil110CDDemultiplexador = simula o demultiplexador do receptor. Sua funcao
%e separar os simbolos PSK de dados do preambulo e dos probes, além de fazer
%o descrambling (desembaralhamento) dos simbolos. Retorna um struct
%'DemultiplexOut' com a seguinte saida:
%
%out = vetor com simbolos PSK, complexos, de dados. Na ausencia de canal,
%deve ser identico ao vetor 'TxOut.ComplexPSKSymbols'. Seu tamanho e
%identico ao de 'ComplexPSKSymbols';
DemultiplexOut   = mil110CDDemultiplexador(SyncSymbolOut.out, mil110CD);

%mil110CDDemodulador = simula o demodulador PSK e o demapeador Gray no
%receptor. Retorna um struct 'DemodulatorOut' com as seguintes saidas:
%
%DemodOutDemapIn = simbolos PSK, inteiros, de saida do demodulador e de
%entrada do demapeador Gray. Seu tamanho e identico ao do vetor
%'DemultiplexOut.out';
%
%DemapOut = 'hard' bits de saida do demapeador Gray, ainda sob efeito de
%emtrelaçamento (interleaving) e codificacao convolucional. Na ausencia de
%canal, deve ser igual ao vetor 'TxOut.InterOutMapIn'. Seu tamanho e
%identico ao de 'InterOutMapIn';
DemodulatorOut   = mil110CDDemodulador(DemultiplexOut.out, mil110CD);

%mil110CDDeinterleaver = simula o deinterleaver no receptor. Retorna um
%struct com a seguinte saida:
%
%out = vetor com bits ('hard', quando a entrada for 'hard' bits; ou 'soft',
%quando a entrada for 'soft' bits) desentrelaçados, mas ainda
%codificados. Na ausencia de canal e quando consistir de 'hard' bits,
%deve ser identico ao vetor 'TxOut.CodOutInterIn'. Seu tamanho e identico
%ao de 'CodOutInterIn';
DeinterleaverOut = mil110CDDeinterleaver(DemodulatorOut.DemapOut, mil110CD);


ber_demod = biterr(TxOut.CodOutInterIn, DeinterleaverOut.out)/length(TxOut.CodOutInterIn);
